/**************************************************************************
 * ESPTEST.C                                                              *
 * Tests ESP enhanced mode receive and transmit                           *
 * Requires configuration file ESPTEST.CFG                                *
 * Requires test file ESPTEST.DAT                                         *
 **************************************************************************/
Note this material was downloaded from the Hayes BBS...
/*** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***
 ***                                                               ***
 ***                  CONFIDENTIAL INFORMATION                     ***
 ***                  ------------------------                     ***
 ***    This Document contains Confidential Information or         ***
 ***    Trade Secrets, or both, which are the property of Hayes    ***
 ***    Microcomputer Products  Inc.  This Document may not be     ***
 ***    copied, reproduced, reduced to any electronic medium or    ***
 ***    machine readable form or otherwise duplicated and the      ***
 ***    information herein may not be used, disseminated or        ***
 ***    otherwise disclosed, except with the prior written consent ***
 ***    of Hayes Microcomputer Products, Inc.                      ***
 ***                                                               ***
 *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***/

/*
    Copyright 1991 Hayes Microcomputer Products, Inc.
    All rights reserved.
    Use of copyright notice does not imply publication.
 */

#include "espfunc.h"
#include <stdio.h>
#include <conio.h>
#include <memory.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>

#define  MAX_COUNT   1024
#define  NONE        0
#define  ECHO        1
#define  LOOPBACK    2

char     *configfile = "esptest.cfg";
FILE     *file;
BYTE     auto_status = 0;
BYTE     sr_mask;
BYTE     dma_time;
BYTE     echo_test[2] = {0, 0};
BYTE     loop_test[2] = {0, 0};
BYTE     flow_byte1[2] = {0, 0};
BYTE     flow_byte2[2] = {0, 0};
BYTE     loop_type[2];
BYTE     rx_dma[2];
BYTE     tx_dma[2];
int      block_size[2];
WORD     baud[2];
BYTE     bits[2];
BYTE     parity[2] = {0, 0};
BYTE     stop[2];
WORD     error_mask[2];
BYTE     rx_time[2];
BYTE     break_time[2];
BYTE     flow_time[2];
BYTE     rx_buffer[2][MAX_COUNT];
BYTE     tx_buffer[2][MAX_COUNT];
int      rx_buf_full[2] = {0, 0};
int      tx_buf_full[2] = {0, 0};
double   pass[2] = {0., 0.};
double   fail[2] = {0., 0.};
char     *testfile = "esptest.dat";
int      result;
int      byte_count;
int      i;   
int      key = '1';
int      rxoff = 0;
int      txoff = 0;
int      quit = 0;

/****************************** LOCAL FUNCTIONS ***************************/

/* ======================= SET_CURSOR_POS ===================================
 * Sets screen cursor position                                        
 */
void set_cursor_pos( row, col )
   unsigned char        row;
   unsigned char        col;
{
   union REGS           regs;

   regs.h.ah = 0x02;              /* function: set cursor position */
   regs.h.bh = 0;                 /* video page 0 */
   regs.h.dh = row;
   regs.h.dl = col;
   int86( 0x10, &regs, &regs );   /* INT 10H is BIOS Video */
}

/* ============================= CLS ========================================
 * Clears entire screen                                               
 */
void cls( )
{
   union REGS          regs;

   regs.h.ah = 0x06;              /* functon: scroll */
   regs.h.al = 0x00;              /* scroll entire screen */
   regs.h.bh = 0x07;              /* normal attribute (black & white) */
   regs.x.cx = 0x00;              /* FROM 00,00 */
   regs.x.dx = 0x184F;            /* TO   24,79 */   
   int86( 0x10, &regs, &regs );   /* INT 10H is BIOS Video */
}

/* ============================ GET_PORT_CONFIG =============================
 * Gets program configuration data for one port from esp.cfg
 * ENTER:       Port = 1 or 2
 * RETURN:      0=OK, 1=file not found, 2=file format error
 */
int   get_port_config( port )
{
   char     string[81];
   BYTE     rx_xx;
   BYTE     txx;
   BYTE     rx_rts;
   BYTE     rx_dtr;
   BYTE     tx_xx;
   BYTE     tx_cts;
   BYTE     tx_dsr;
   int      temp;
   
   result = fscanf( file, "   Test (ECHO, LOOPBACK, or NONE): %s", string );
   if(result != 1)  return( 2 );
   if( strcmpi( string, "echo" ) == 0 ) echo_test[port-1] = 1;
   if( strcmpi( string, "loopback" ) == 0 ) loop_test[port-1] = 1;
   
   result = fscanf( file, " Loopback type (WIRE or INTERNAL): %s", string );
   if(result != 1)  return( 2 );
   if( strcmpi( string, "wire" ) == 0 ) loop_type[port-1] = 0;
   if( strcmpi( string, "internal" ) == 0 ) loop_type[port-1] = LOOP;

   result = fscanf( file, 
      " RX Flow (x x x x), x=0 or 1 for XX, TXX, RTS, DTR: %d %d %d %d", 
      &rx_xx, &txx, &rx_rts, &rx_dtr );
   if(result != 4)  return( 2 );
   result = fscanf( file, 
      " TX Flow (x x x x), x=0 or 1 for XX, TXX, CTS, DSR: %d %d %d %d", 
      &tx_xx, &txx, &tx_cts, &tx_dsr );
   if(result != 4)  return( 2 );
   flow_byte1[port-1] = rx_xx | (txx<<1) | (rx_rts<<2) | (rx_dtr<<3) | (tx_xx<<7);
   flow_byte2[port-1] = (tx_cts<<4) | (tx_dsr<<5);
   
   result = fscanf( file, " RX transfer type (IO or DMA): %s", string );
   if(result != 1)  return( 2 );
   if( strcmpi( string, "io" ) == 0 ) rx_dma[port-1] = CHAR_RX;
   if( strcmpi( string, "dma" ) == 0 ) rx_dma[port-1] = DMA_RX;
 
   result = fscanf( file, " TX transfer type (IO or DMA): %s", string );
   if(result != 1)  return( 2 );
   if( strcmpi( string, "io" ) == 0 ) tx_dma[port-1] = CHAR_TX;
   if( strcmpi( string, "dma" ) == 0 ) tx_dma[port-1] = DMA_TX;
    
   result = fscanf( file, " Transfer block size (1 - 1023): %d", 
      &block_size[port-1] );
   if(result != 1)  return( 2 );
   
   result = fscanf( file, " Baud: %d", &temp );
   baud[port-1] = (WORD)temp;
   if(result != 1)  return( 2 );
   
   result = fscanf( file, " Data bits (5, 6, 7, or 8): %d", &temp );
   bits[port-1] = (BYTE)temp;
   if(result != 1)  return( 2 );
   
   result = fscanf( file, " Parity (NONE, ODD, EVEN, MARK, or SPACE): %s", 
   string );
   if(result != 1)  return( 2 );
   if( strcmpi( string, "none" ) == 0 ) parity[port-1] = 0;
   if( strcmpi( string, "odd" ) == 0 ) parity[port-1] = 1;
   if( strcmpi( string, "even" ) == 0 ) parity[port-1] = 2;
   if( strcmpi( string, "mark" ) == 0 ) parity[port-1] = 3;
   if( strcmpi( string, "space" ) == 0 ) parity[port-1] = 4;
   
   result = fscanf( file, " Stop bits (1, 1.5, or 2): %s", string );
   if(result != 1)  return( 2 );
   if( strcmpi( string, "1" ) == 0 ) stop[port-1] = 0;
   if( strcmpi( string, "1.5" ) == 0 ) stop[port-1] = 1;
   if( strcmpi( string, "2" ) == 0 ) stop[port-1] = 2;

   result = fscanf( file, " Error Status Mask (hex): %x", &temp );
   error_mask[port-1] = (WORD)temp;
   if(result != 1)  return( 2 );
   
   result = fscanf( file, " Receive timeout (ms): %d", &temp );
   rx_time[port-1] = (BYTE)temp;
   if(result != 1)  return( 2 );
   
   result = fscanf( file, " Break duration (ms): %d", &temp );
   break_time[port-1] = (BYTE)(temp / 10);
   if(result != 1)  return( 2 );
   
   result = fscanf( file, " Flowed off timeout (s): %d", &temp );
   flow_time[port-1] = (BYTE)temp;
   if(result != 1)  return( 2 );
   
   return( 0 );
}

/* ================================ GETCONFIG =============================
 * Gets program configuration data from esp.cfg
 * ENTER:       None
 * RETURN:      0=OK, 1=file not found, 2=file format error
 */
int   getconfig()
{
   char     string[81];
   int      temp;

   if( ( file = fopen( configfile, "rb" ) ) == NULL )
      return( 1 );

   result = fscanf( file, " Computer type (AT or PS2): %s", string );
   if(result != 1)  return( 2 );
   if( strcmpi( string, "at" ) == 0 ) computer = AT;
   if( strcmpi( string, "ps2" ) == 0 ) computer = PS2;

   result = fscanf( file, " Enhanced address (hex): %x", &temp );
   enhanced_base = (WORD)temp;
   if(result != 1)  return( 2 );
   
   result = fscanf( file, " DMA channel (1 or 3): %d", &temp );
   dmachan = (BYTE)temp;
   if(result != 1)  return( 2 );
   
   result = fscanf( file, " Enhanced IRQ ( 9, 3, 4, 5, or DISABLED): %s", string );
   if(result != 1)  return( 2 );
   if( strcmpi( string, "9" ) == 0 ) irq = 1;
   if( strcmpi( string, "3" ) == 0 ) irq = 3;
   if( strcmpi( string, "4" ) == 0 ) irq = 5;
   if( strcmpi( string, "5" ) == 0 ) irq = 7;
   if( strcmpi( string, "disabled" ) == 0 ) irq = 0;

   result = fscanf( file, " Automatically Update Status (Y or N): %s", string );
   if(result != 1)  return( 2 );
   if( strcmpi( string, "Y" ) == 0 ) auto_status = 1;

   result = fscanf( file, " Service Request Mask (hex): %x", &temp );
   sr_mask = (BYTE)temp;
   if(result != 1)  return( 2 );
   
   result = fscanf( file, " DMA timeout (ms): %d", &temp );
   dma_time = (BYTE)(temp / 10);
   if(result != 1)  return( 2 );
   
   fscanf( file, " Port 1 stuff: " );
   result = get_port_config(1);
   if( result )  return( result);

   fscanf( file, " Port 2 stuff: " );
   result = get_port_config(2);
   if( result )  return( result);

   return( 0 );
}

/* ============================= GETPATTERN =============================
 * Gets test pattern from file esp.tst
 * ENTER:       None
 * RETURN:      1 if file OK, 0 if file not OK
 */
int   getpattern()
{
   if( loop_test[0] )
   {
      if( ( file = fopen( testfile, "rb" ) ) == NULL )
      {
         printf( "Need test file %s\n", testfile );
         return(0);
      }
      byte_count = fread( tx_buffer[0], sizeof( BYTE ), MAX_COUNT, file );
      fclose( file );
      if( byte_count < block_size[0] )
      {
         printf( "Test file %s is too small\n", testfile );
         return(0);
      }
   }
   if( loop_test[1] )
   {
      if( ( file = fopen( testfile, "rb" ) ) == NULL )
      {
         printf( "Need test file %s\n", testfile );
         return(0);
      }
      byte_count = fread( tx_buffer[1], sizeof( BYTE ), MAX_COUNT, file );
      fclose( file );
      if( byte_count < block_size[1] )
      {
         printf( "Test file %s is too small\n", testfile );
         return(0);
      }
   }
   return(1);
}

/* ================================ SETUP =============================
 * Does port-specific setup stuff
 * ENTER:       Port = 1 or 2
 * RETURN:      None
 */
void setup( port )
BYTE     port;
{
   if( echo_test[port-1] || loop_test[port-1] )
   {
      set_port( port );
      seterrormsk( error_mask[port-1] );
      setfctyp( flow_byte1[port-1] | RTS | DTR | loop_type[port-1],
                flow_byte2[port-1] );
      setfcchar( 0x11, 0x13, 0x10, 0x21, 0x7F );
      setflolev( 768, 512 );
      settrig( block_size[port-1], block_size[port-1] );
      setrxtime( rx_time[port-1] );
      setflotime( flow_time[port-1] );
      set_baud_rate( baud[port-1] );
      set_line_characteristics( bits[port-1], parity[port-1], stop[port-1] );
      setpmode( (BYTE)ENHANCED | rx_dma[port-1] | tx_dma[port-1] );
   }
}

/* ================================ STATUS =============================
 * Writes ESP status & pass/fail count to console
 * ENTER:       None
 * RETURN:      None
 */
void  status()
{
   set_cursor_pos( 0, 0 );
   printf( "                      STATUS:\n" );
   printf( "\n" );
   printf( "     ready register =   %.2X hex\n", inp( register_ready ) );
   printf( "         service ID =   %.2X hex\n", inp( service_id ) );
   printf( "   self test result = %.4X hex\n", gettest() );
   printf( "       address DIPs =   %.2X hex\n", getdips() );
   printf( "\n" );
   set_port( 1 );
   printf( "        port 1 mode = %.4X hex\n", getmode() );
   set_port( 2 );
   printf( "        port 2 mode = %.4X hex\n", getmode() );
   set_port( 1 );
   printf( "port 1 error status = %.4X hex\n", geterrs() );
   set_port( 2 );
   printf( "port 2 error status = %.4X hex\n", geterrs() );
   set_port( 1 );
   printf( " port 1 UART status = %.4X hex\n", getuarts() );
   set_port( 2 );
   printf( " port 2 UART status = %.4X hex\n", getuarts() );
   set_port( 1 );
   printf( "    port 1 UART LCR = %.2X hex\n", readuart( 3 ) );
   set_port( 2 );
   printf( "    port 2 UART LCR = %.2X hex\n", readuart( 3 ) );
   set_port( 1 );
   printf( "    port 1 RX bytes = %d    \n", getrxbytes() );
   set_port( 2 );
   printf( "    port 2 RX bytes = %d    \n", getrxbytes() );
   set_port( 1 );
   printf( "    port 1 TX space = %d    \n", gettxbytes() );
   set_port( 2 );
   printf( "    port 2 TX space = %d    \n", gettxbytes() );
   printf( "  port 1 bad blocks = %.0f    \n", fail[0] );
   printf( "  port 2 bad blocks = %.0f    \n", fail[1] );
   printf( " port 1 good blocks = %.0f    \n", pass[0] );
   printf( " port 2 good blocks = %.0f    \n", pass[1] );
}

/* ================================ SERVICE_KEY =============================
 * Services keyboard input
 * ENTER:       None
 * RETURN:      None
 */
void service_key()
{
   if( key == '1' )        /* send xoff */
   {
      set_port( 1 );
      writeuart( 0, 0x13 );
      set_port( 2 );
      writeuart( 0, 0x13 );
   }
   if( key == '2' )        /* send xon */
   {
      set_port( 1 );
      writeuart( 0, 0x11 );
      set_port( 2 );
      writeuart( 0, 0x11 );
   }
   if( key == '3' )        /* stop reading RX data */
      rxoff = 1;
   if( key == '4' )        /* start reading RX data */
      rxoff = 0;
   if( key == '5' )        /* issue xoff local */
   {
      set_port( 1 );
      xofflocal();
      set_port( 2 );
      xofflocal();
   }
   if( key == '6' )        /* issue xon local */
   {
      set_port( 1 );
      xonlocal();
      set_port( 2 );
      xonlocal();
   }
   if( key == '7' )        /* stop reading TX data */
      txoff = 1;
   if( key == '8' )        /* start reading TX data */
      txoff = 0;
   if( key == 'S' || key == 's' )      /* update status */
      status();
   if( key == 'B' || key == 'b' )      /* send break */
   {
      set_port( 1 );
      linebreak( break_time[0] );
      set_port( 2 );
      linebreak( break_time[1] );
   }
   if( key == 'R' || key == 'r' )      /* flush RX FIFO */
   {
      set_port( 1 );
      flushrx();
      set_port( 2 );
      flushrx();
   }
   if( key == 'T' || key == 't' )      /* flush TX FIFO */
   {
      set_port( 1 );
      flushtx();
      set_port( 2 );
      flushtx();
   }
   if( key == 'Q' || key == 'q' )      /* quit esp */
      quit = 1;
}

 /* ================================ WAIT_TC =============================
 * Waits til DMA TC reached or 'd' key hit
 * Disables DMA xfers if 'd' key hit
 * ENTER:       None
 * RETURN:      None
 */
void wait_tc()
{
   BYTE  service;

   while( 1 )
   {
      if( kbhit() )
      {
         key = getch();
         if( key != 'D' && key  != 'd' )
            service_key();
         else
         {
            if( computer == AT )
               outp( 0x0A, 0x04+dmachan );   /* disable dma xfers */
            if( computer == PS2 )
               outp( 0x18, 0x90+dmachan );   /* disable dma xfers */
            return;
         }
      }
      service = (BYTE)inp(service_id);
      if( service & DMA_TC_REACHED ) 
         return;
      if( service & DMA_TIMEOUT )
      {
         status();
         return;
      }
   }
}

/* ================================ RECEIVE =============================
 * Receives (block_size) bytes from RX port & stores them in rx_buffer[]
 * ENTER:       Port = 1 or 2
 * RETURN:      None
 */
void receive( port )
BYTE     port;
{
   int   i;

   set_port( port );
   if( rx_dma[port-1] )
   {  
      initiate_dma( rx_buffer[port-1], block_size[port-1], WRITE );
      dmarx();
      wait_tc();
   }
   else
   {
      for( i=0; i<block_size[port-1]; i++ )
         rx_buffer[port-1][i] = rxbyte();
   }
}

/* ================================ TRANSMIT =============================
 * Gets (block_size) bytes from buffer[] & transmits them to TX port
 * ENTER:       Port = 1 or 2
 * RETURN:      None
 */
void transmit( port )
BYTE     port;
{
   int   i;

   set_port( port );
   if( tx_dma[port-1] )
   {  
      initiate_dma( tx_buffer[port-1], block_size[port-1], READ );
      dmatx();
      wait_tc();
   }
   else
   {
      for( i=0; i<block_size[port-1]; i++ )
         txbyte( tx_buffer[port-1][i] );
   }
}

/* ================================ ECHO =============================
 * Receives (block_size) bytes from RX port & writes them to TX port
 * ENTER:       Port = 1 or 2
 * RETURN:      None
 */
void echo( port )
BYTE     port;
{
   set_port( port );
   if( ((BYTE) inp( service_id ) & rx_fifo) && !rxoff && !rx_buf_full[port-1] )
   {
      receive( port );
      pass[port-1]++;
      set_cursor_pos( 21+port-1, 22 );
      printf( "%.0f", pass[port-1] );
   }
   if( rx_buf_full[port-1] && !tx_buf_full[port-1] )
   {
      memcpy( tx_buffer[port-1], rx_buffer[port-1], block_size[port-1] );
      rx_buf_full[port-1] = 0;
      tx_buf_full[port-1] = 1;
   }
   if( ((BYTE)inp( service_id ) & tx_fifo) && !txoff && tx_buf_full[port-1] )
      transmit( port );
}

/* ================================ LOOP =============================
 * Writes (block_size) bytes to TX port, & reads & compares from RX port
 * ENTER:       Port = 1 or 2
 * RETURN:      None
 */
void loop( port )
BYTE     port;
{
   set_port( port );
   if( ((BYTE)inp( service_id ) & tx_fifo) && !txoff )
      transmit( port );
   if( ((BYTE)inp( service_id ) & rx_fifo) && !rxoff )
   {
      memset( rx_buffer[port-1], '\0', block_size[port-1] );
      receive( port );
      if( memcmp( rx_buffer[port-1], tx_buffer[port-1], block_size[port-1] ) == 0)
      {
         pass[port-1]++;
         set_cursor_pos( 21+port-1, 22 );
         printf( "%.0f", pass[port-1] );
      }
      else
      {
         fail[port-1]++;
         set_cursor_pos( 19+port-1, 22 );
         printf( "%.0f", fail[port-1] );
         flushtx();
         flushrx();
      }
   }
}

/********************************** ESP *******************************/

void main()
{
   /* GET CONFIGURATION FROM FILE */
   result = getconfig();
   if( result == 1 )
   {
      printf( "Need configuration file %s\n", configfile ); 
      return; 
   }
   if(result == 2) 
   {
      printf("%s file format error\n", configfile); 
      return;
   }

   /* GET TEST PATTERN FROM FILE */
   if( !getpattern() ) return;

   cls();
   set_cursor_pos( 0, 0 );
   printf(   "                                        KEYS AVAILABLE:" );
   printf( "\n" );
   printf( "\n                                        1 - send XOFF to remote" );
   printf( "\n                                        2 - send XON to remote" );
   printf( "\n                                        3 - stop reading RX data" );
   printf( "\n                                        4 - start reading RX data" );
   printf( "\n                                        5 - issue XOFF local" );
   printf( "\n                                        6 - issue XON local" );
   printf( "\n                                        7 - stop sending TX data" );
   printf( "\n                                        8 - start sending TX data" );
   printf( "\n" );
   printf( "\n                                        S - update status" );
   printf( "\n                                        D - abort DMA transfer" );
   printf( "\n                                        B - send break" );
   printf( "\n                                        R - flush RX FIFO" );
   printf( "\n                                        T - flush TX FIFO" );
   printf( "\n" );
   printf( "\n                                        Q - quit esp" );

   /* SETUP */
   init();
   status();
   reset ();
   setirq( (dmachan << 4) | irq );
   setdmatime( dma_time );
   if( echo_test[0] || loop_test[0] )
      setup( 1 );
   if( echo_test[1] || loop_test[1] )
      setup( 2 );
   setsrmsk( sr_mask );
   status();

   /* DO LOOP OR ECHO TESTS */
   while ( quit != 1 )
   {
      if( echo_test[0] )   
         echo( 1 );
      if( loop_test[0] )
         loop( 1 );
      if( echo_test[1] )   
         echo( 2 );
      if( loop_test[1] )
         loop( 2 );
      if( auto_status )
      {
         status();
         setirq( (dmachan << 4) | irq );
         setdmatime( dma_time );
         if( echo_test[0] || loop_test[0] )
            setup( 1 );
         if( echo_test[1] || loop_test[1] )
            setup( 2 );
         setsrmsk( sr_mask );
      }
      if( kbhit() )
      {
         key = getch();
         service_key();
      }
   }
   reset();
   set_cursor_pos( 23, 0 );
}

 
